<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Nov 27, 2020
 */

/**
 * Class that determines mountpoint by path.
 *
 * @package Interworx
 * @subpackage Plugin_Acronis
 */
class Model_RecoveryPoint_MountPoint
{
    private $_isDefault;
    private $_path;
    private $_mountpoint;

    /**
     * @param string $path
     */
    public function __construct($path)
    {
        $this->_path = $path;

        if ($this->pathStartsWith('chroot')) {
            $this->_mountpoint = $this->_getChrootMountpoint();
        }

        if ($this->pathStartsWith('home')) {
            $this->_mountpoint = $this->_getHomeMountpoint();
        }

        if (!isset($this->_mountpoint)) {
            $this->_mountpoint = $this->_getDefaultMountpoint();
        }
    }

    /**
     * check if path (haystack) starts with given needle.
     *
     * @param string $startsWith
     * @return boolean
     */
    public function pathStartsWith($startsWith)
    {
        $pos = strpos($this->_path, $startsWith);
        return $pos !== false && ($pos === 0 || $pos === 1);
    }

    /**
     * Get mountpoint for /home
     *
     * @return string
     */
    private function _getHomeMountpoint()
    {
        $cmd = "mount | grep \"on $(dirname $(readlink -f /home))\" | awk '{print $1}'";

        $Command = new Command($cmd);
        $Command->allowPipe();
        $Command->allowParenthesisBoth();
        $Command->allowQuoteSingle();
        $Command->allowQuoteDouble();
        $Command->allowCurlyBoth();
        $Command->allowDollarSign();
        $Command->exec();

        $mpDevPath = $Command->getOutput()[0];

        $this->_isDefault = empty($mpDevPath);

        $mp = empty($mpDevPath) ? $this->_getDefaultMountpoint() :  $this->_getLastFromPath($mpDevPath);
        return $mp;
    }

    /**
     * Get mountpoint for /chroot
     *
     * @return string
     */
    private function _getChrootMountpoint()
    {
        $chrootPath = $this->_getDevMountpointPath('/chroot');
        $this->_isDefault = empty($chrootPath);

        $mp = empty($chrootPath) ? $this->_getDefaultMountpoint() :  $this->_getLastFromPath($chrootPath);
        return $mp;
    }

    /**
     * Get default mp for /
     *
     * @return string
     */
    private function _getDefaultMountpoint()
    {
        $this->_isDefault = true;

        $defaultPath = $this->_getDevMountpointPath('/');
        return $this->_getLastFromPath($defaultPath);
    }

    /**
     * Get dev mountpoint path.
     *
     * @param string $path
     * @return string
     */
    private function _getDevMountpointPath($path)
    {
        $Command = new Command("mount | grep 'on $path ' | awk '{print $1}'");
        $Command->allowPipe();
        $Command->allowQuoteSingle();
        $Command->allowCurlyBoth();
        $Command->allowDollarSign();
        $Command->exec();

        return $Command->getOutput()[0];
    }

    /**
     * get last dir/file from path
     *
     * @param string $path
     * @return string
     */
    private function _getLastFromPath($path)
    {
        if (empty($path)) {
            throw new Exception('mountpoint path is empty');
        }

        $dirs = explode('/', $path);
        return end($dirs);
    }

    /**
     * Check if mp is default (/).
     *
     * @return boolean
     */
    public function isDefault()
    {
        return $this->_isDefault === true;
    }

    /**
     * Check if mp is other than /
     *
     * @return boolean
     */
    public function getMountpoint()
    {
        return $this->_mountpoint;
    }

    /**
     * Check if path is chroot.
     *
     * @return boolean
     */
    public function pathIsChroot()
    {
        return $this->_path === 'chroot' || $this->_path === '/chroot';
    }
}
