<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Apr 20, 2020
 */

/**
 * Resource class.
 *
 * @package Interworx
 * @subpackage Plugin
 */
class Model_Server_Server extends Model_Acronis
{
    const NEXTBACKUP = 'nextBackup';
    const LASTBACKUP = 'lastBackup';
    const LASTSTATUS = 'lastStatus';
    const RUNNING = 'running';

    private $_resourceId;
    private $_instanceId;

    private $_lastBackup;
    private $_nextBackup;
    private $_online;
    private $_status;


    /**
     * Get plan id.
     *
     * @return string
     * @throws Exception // if no plan is assigned.
     */
    public function getPlanId()
    {
        $planId = $this->_getAssignedPlanId();
        return $planId;
    }

    /**
     * Get id from any of the already assigned plans.
     *
     * A resource can have multiple backup plans.
     * Look for plan with valid pre/post script,
     * and return any if no plans with valid scripts
     * exists. The daily cron will detect invalid
     * plans and replace them.
     *
     * @return string
     */
    private function _getAssignedPlanId()
    {
        $Api = $this->_getApi();
        $plans = $Api->getBackupPlans($this->getInstanceId());

        if (empty($plans)) {
            throw new Exception_BackupPlan_NoneAssigned('no plans are assigned to the resource.');
        }

        $validatedPlan = $this->_getPlanWithValidScripts($plans);

        // get any plan if none found with valid scripts
        $plan = empty($validatedPlan) ? $plans[0] : $validatedPlan;
        return $plan['planId'];
    }

    /**
     * return plan if scripts are ok.
     *
     * @param array $plans
     * @return array
     */
    private function _getPlanWithValidScripts(array $plans)
    {
        foreach ($plans as $plan) {
            $planId = $plan['planId'];
            $planOk = $this->_prePostScriptsOk($planId);

            if ($planOk) {
                return $plan;
            }
        }
    }

    /**
     * Get applicable plan id.
     *
     * @return string
     * @throws Exception // if no applicable plans are found.
     */
    public function getApplicablePlanId()
    {
        $Api = $this->_getApi();
        $applicablePlans = $Api->getApplicablePlans($this->getResourceId())['Plans'];

        foreach ($applicablePlans as $plan) {
            $planId = $plan['ID'];

            if ($this->_prePostScriptsOk($planId)) {
                return $planId;
            }
        }

        throw new Exception_BackupPlan_NoApplicablePlans('No applicable plan was found!');
    }

    /**
     * Make sure pre/post scripts are enabled and configured.
     *
     * @param string $planId
     */
    protected function _prePostScriptsOk($planId)
    {
        $BackupPlan = $this->_getPlugin()->getBackupPlan($planId);
        return $BackupPlan->prePostScriptsOK();
    }

    /**
     * Get resource backup status. includes last backup, next backup, last backup status.
     *
     * @return array
     */
    public function getBackupStatus()
    {
        $Api = $this->_getApi();
        $instanceId = $this->getInstanceId();

        $apiRes = $Api->getBackupPlans($instanceId);
        $status = $this->_formatBackupStatus($apiRes[0]);

        return $status;
    }

    /**
     * Get next backup.
     *
     * @return boolean
     */
    public function backupNotScheduled()
    {
        if ($this->isProtected()) {
            $backupStatus = $this->getBackupStatus();
            $nextBackup = $backupStatus[self::NEXTBACKUP];
            return empty($nextBackup);
        }

        return true;
    }

    /**
     * check if backup in progress.
     */
    public function backupInProgress()
    {
        $backupStatus = $this->getBackupStatus();
        $lastStatus = $backupStatus[self::LASTSTATUS];
        return $lastStatus === self::RUNNING;
    }

    /**
     * Format backup status object.
     *
     * @param array $backupStatus
     * @return array
     */
    private function _formatBackupStatus(array $backupStatus)
    {
        return array(
            self::LASTSTATUS => $backupStatus['lastStatus'],
            self::LASTBACKUP => $this->_formatDate($backupStatus['lastStartTime']),
            self::NEXTBACKUP => $this->_formatDate($backupStatus['nextStartTime'])
        );
    }


    /**
     * Get resource id.
     *
     * @return string
     */
    public function getResourceId()
    {
        if (!isset($this->_resourceId)) {
            $this->_setResourceIdFromCacheOrApi();
        }

        return $this->_resourceId;
    }

    /**
     * set resource id either from cached value or by api request.
     */
    private function _setResourceIdFromCacheOrApi()
    {
        $this->_resourceId = $this->_getCache()->getResourceId();

        if (!isset($this->_resourceId)) {
            $this->_setDeviceInfo();
        }
    }

    /**
     * Get instance id.
     *
     * @return string
     */
    public function getInstanceId()
    {
        if (!isset($this->_instanceId)) {
            $this->_setInstanceIdFromCacheOrApi();
        }

        return $this->_instanceId;
    }

    /**
     * Set instance id either from cache value or by api req.
     */
    private function _setInstanceIdFromCacheOrApi()
    {
        $this->_instanceId = $this->_getCache()->getInstanceId();

        if (!isset($this->_instanceId)) {
            $this->_setDeviceInfo();
        }
    }

    /**
     * Get last date string.
     *
     * @return string
     */
    public function getLastBackup()
    {
        if (!isset($this->_lastBackup)) {
            $this->_setDeviceInfo();
        }

        return $this->_formatDate($this->_lastBackup);
    }

    /**
     * Get next backup date string.
     *
     * @return string
     */
    public function getNextBackup()
    {
        if (!isset($this->_nextBackup)) {
            $this->_setDeviceInfo();
        }

        return $this->_formatDate($this->_nextBackup);
    }


    /**
     * Check if machine is online.
     *
     * @return boolean
     */
    public function isOnline()
    {
        if (!isset($this->_online)) {
            $this->_setDeviceInfo();
        }

        return $this->_online;
    }

    /**
     * Get last backup status.
     *
     * @return string
     */
    public function isProtected()
    {
        $this->_setDeviceInfo();
        return $this->_status !== 'not_protected';
    }

    /**
     * Get device storage size.
     *
     * @return string
     */
    public function getStorageData()
    {
        if (!isset($this->_instanceId)) {
            $this->_setDeviceInfo();
        }

        $Api = $this->_getApi();
        $storageInfo = $Api->getStorageData($this->_instanceId);
        $gb = $this->_convertBytes($storageInfo['occupied_data_size']);
        return array('size' => $gb);
    }


    /**
     * Set device info.
     */
    private function _setDeviceInfo()
    {
        $device = $this->_getCache()->getDeviceInfo();

        if (empty($device)) {
            $Api = $this->_getApi();
            $apiResponse = $Api->getDevice();
            $device = $this->_formatDeviceInfoFromApi($apiResponse);
            $this->_getCache()->cacheDeviceInfo($device);
        }

        $this->_resourceId = $device['resourceId'];
        $this->_lastBackup = $device['lastBackup'];
        $this->_nextBackup = $device['nextBackup'];
        $this->_online = $device['online'];
        $this->_instanceId = $device['instanceId'];
        $this->_status = $device['status'];

        $this->_getCache()->cacheResourceId($this->_resourceId);
        $this->_getCache()->cacheInstanceId($this->_instanceId);
    }

    /**
     *
     * Format device info from api.
     *
     * @param array $deviceInfo
     * @return array
     */
    private function _formatDeviceInfoFromApi(array $deviceInfo)
    {
        return array(
            'resourceId' => $deviceInfo['id'],
            'lastBackup' => $deviceInfo['lastBackup'],
            'nextBackup' => $deviceInfo['nextBackup'],
            'online' => $deviceInfo['online'],
            'instanceId' => $deviceInfo['instanceId'],
            'status' => $deviceInfo['status']
        );
    }
}
